// our precalculated mandelbrot fractals
unsigned char *frac1, *frac2, *fractmp;

// define the point in the complex plane to which we will zoom into
#define or_cplx    -0.577816-9.31323E-10-1.16415E-10
#define oi_cplx    -0.631121-2.38419E-07+1.49012E-08

/*
 * global variables used for calculating our fractal
 */
double dr, di, pr, pi, sr, si;
long offs;
bool zoom_in;
double zx, zy;

/*
 * initialize the fractal computation
 */
void Start_Frac( double _sr, double _si, double er, double ei )
{
// compute deltas for interpolation in complex plane
   dr = (er-_sr)/640.0f;
   di = (ei-_si)/400.0f;
// remember start values
   pr = _sr;
   pi = _si;
   sr = _sr;
   si = _si;
   offs = 0;
}

/*
 * compute 4 lines of fracal
 */
void Compute_Frac()
{
   for (int j=0; j<4; j++)
   {
       pr = sr;
       for (int i=0; i<640; i++)
       {
           unsigned char c = 0;
           double vi = pi, vr = pr, nvi, nvr;
        // loop until distance is above 2, or counter hits limit
           while ((vr*vr + vi*vi<4) && (c<255))
           {
              // compute Z(n+1) given Z(n)
                 nvr = vr*vr - vi*vi + pr;
                 nvi = 2 * vi * vr + pi;

             // that becomes Z(n)
                 vi = nvi;
                 vr = nvr;

             // increment counter
                 c++;
           }
        // store colour
           frac1[offs] = c;
           offs++;
        // interpolate X
           pr += dr;
       }
    // interpolate Y
       pi += di;
   }
}

/*
 * finished the computation, swap buffers
 */
void Done_Frac()
{
   fractmp = frac1;
   frac1 = frac2;
   frac2 = fractmp;
}

/*
 * the zooming procedure
 * takes a 640x400 bitmap, and scales it to 320x200 given the zooming coef
 */
void Zoom( double z )
{
  // what's the size of rectangle in the source image we want to display
     int width = (int)((640<<16)/(256.0f*(1+z)))<<8,
         height = (int)((400<<16)/(256.0f*(1+z)))<<8,
  // where do we start our interpolation
         startx = ((640<<16)-width)>>1,
         starty = ((400<<16)-height)>>1,
  // get our deltas
         deltax = width/320,
         deltay = height/200,
         px, py = starty;
     long offs = 0;
     vga->Lock();
     Uint8 *page_draw = vga->GetSurfaceData();
     for (int j=0; j<200; j++)
     {
      // set start value
         px = startx;
         for (int i=0; i<320; i++)
         {
          // load the bilinear filtered texel
             page_draw[offs] =
             (    frac2[(py>>16)*640+(px>>16)] * (0x100-((py>>8)&0xff)) * (0x100-((px>>8)&0xff))
                + frac2[(py>>16)*640+((px>>16)+1)] * (0x100-((py>>8)&0xff)) * ((px>>8)&0xff)
                + frac2[((py>>16)+1)*640+(px>>16)] * ((py>>8)&0xff) * (0x100-((px>>8)&0xff))
                + frac2[((py>>16)+1)*640+((px>>16)+1)] * ((py>>8)&0xff) * ((px>>8)&0xff) ) >> 16;
          // interpolate X
             px += deltax;
             offs++;
         }
      // interpolate Y
         py += deltay;
     }
     vga->Unlock();
}

/*
 * calculate some weird colours to make things look less boring
 */
void updatePalette( int t )
{
  const float PI = 1.1415f;
    for (int i=0; i<256; i++)
    {
        vga->SetColour( i,
           (unsigned char)(32 - 31 * cos( i * PI / 128 + t*0.00141 )),
           (unsigned char)(32 - 31 * cos( i * PI / 128 + t*0.00141 )),
           (unsigned char)(32 - 31 * cos( i * PI / 64 + t*0.0136 )) );
    }
}

void Zoom_Init()
{
    int i, j, k = 0;
// allocate memory for our fractal
    frac1 = new (unsigned char)[256000];
    frac2 = new (unsigned char)[256000];
// set original zooming settings
//  double zx = 2.91038E-11, zy = 2.91038E-11;
//  bool zoom_in = false;
    zx = 4.0;
    zy = 4.0;
    zoom_in = true;
// calculate the first fractal
    Start_Frac( or_cplx-zx, oi_cplx-zy, or_cplx+zx, oi_cplx+zy );
    for (j=0; j<100; j++) Compute_Frac();
    Done_Frac();
}

void Zoom_Run()
{
//
    int j, k;
    updatePalette( 0 );
    bQuit = false;
// run the main loop
    while (!kbhit() && !bQuit)
    {
      // adjust zooming coefficient for next view
         if (zoom_in)
         {
            zx *= 0.5;
            zy *= 0.5;
         }
         else {
            zx *= 2;
            zy *= 2;
         }
      // start calculating the next fractal
         Start_Frac( or_cplx-zx, oi_cplx-zy, or_cplx+zx, oi_cplx+zy );
         j = 0;
         while ((j<100) && (!kbhit()) && !bQuit)
         {
             j++;
          // calc another few lines
             Compute_Frac();
          // display the old fractal, zooming in or out
             if (zoom_in) Zoom( (double)j/100.0f );
             else Zoom( 1.0f-(double)j/100.0f );
          // select some new colours
             updatePalette( k*100+j );
          // dump to screen
             vga->Update();
             frameCount++;
         }
      // one more image displayed
         k++;
      // check if we've gone far enough
         if (k%38==0)
         {
         // if so, reverse direction
            zoom_in = !zoom_in;
            if (zoom_in) {
               zx *= 0.5;
               zy *= 0.5;
            }
            else {
               zx *= 2.0;
               zy *= 2.0;
            }
         // and make sure we use the same fractal again, in the other direction
            fractmp = frac1;
            frac1 = frac2;
            frac2 = fractmp;
         }
         Done_Frac();
    }
}

void Zoom_Done()
{
    delete frac1;
    delete frac2;
}
